//======================================================= file = in_sel.c =====
//=  Jitter controller for IYER PPS (distributed) switch simulation model     =
//=============================================================================
//=  Notes:  1) Implements jitter control mechanism at input ports of	      =	 
//=             internal switch planes	                                      =	
//=---------------------------------------------------------------------------=
//=  Contains: in_sel()                                                       =	
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

void in_sel(int channel_num)
{
 int j;      // Generic loop counter
 int count;  // Counter variable, to count # of eligible cells      
 long tag;   // Arrival timestamp of HOL cell in inport buffers 
 int flag[N]; // Boolean flag
 int first_flag; // Boolean flag 
 int temp1[N];   // Temporary variables
 int temp2[N];   // Temporary variables
 int minimum;    // Minimum timestamp of a cell among the HOL cells
 int min_port;   // Input port # of a cell (having minimum timestamp)

 create("in_sel");

 // Initializing the variables
 for(j=0;j<N;j++)
 {
   flag[j] = FALSE;
   temp1[j]= 0;
   temp2[j] = FALSE;
 }

 // Infinite Loop
 while(TRUE)
 {
   // Hold for an internal time slot
   hold(K * CELL_TIME);

   // Receving  arrival timestamp of HOL cells (from inport buffers)
   count = 0;
   for(j=0;j<N;j++)
   {
     if((queue_cnt(In_Q_event[channel_num][j])>0)&&(flag[j]==FALSE))
     {
       receive(Tag_number[channel_num][j], (long *) &tag);
       temp1[j] = tag;
       flag[j] = TRUE;
     }

     // Delaying the HOL cell to the maximum delay (K*N) in demultiplexor
     if((flag[j] == TRUE)&&(clock >= (temp1[j] + K*N)))
     {
       temp2[j] = TRUE;
       count++;
     }
   }

   // Loop till all eligible cells are switched
   while(count>0)
   {
     first_flag = TRUE;
 
     // Finding Cell with minimum timestamp
     for(j=0;j<N;j++)
     {
       if((temp2[j]==TRUE)&&(first_flag==TRUE))
       {
         minimum = temp1[j];
         min_port = j;
         first_flag = FALSE;
       }
       else
       {
         if((temp2[j]==TRUE)&&(temp1[j]<minimum))
         {
           minimum = temp1[j];
           min_port = j;
         }
       }
     }

     // Transmit the selected cell 
     set(In_Q_event[channel_num][min_port]);
     flag[min_port] = FALSE;
     temp2[min_port] = FALSE;
     count--;
     hold(0.0);

     // Receving  arrival timestamp of HOL cells (from inport buffers)
     if(queue_cnt(In_Q_event[channel_num][min_port])>0)
     {
       receive(Tag_number[channel_num][min_port], (long *) &tag);
       temp1[min_port] = tag;
       flag[min_port] = TRUE;
     }

     // Delaying the HOL Cell till the maximum delay (K*N) in Demultiplexor
     if((clock >= (temp1[min_port] + K*N))&&(flag[min_port]==TRUE))
     {
       temp2[min_port] = TRUE;
       count++;
     }
   }
 }
}
